import numpy as np
import skyfield as sf
from astropy.time import Time
from skyfield.api import Topos, load


# Skyfield Timescale
try:
    ts = load.timescale()
except:
    ts = load.timescale(builtin=True)

# Position of MWA site in Lat/Lon/Elevation
MWA = Topos(latitude=-26.703319, longitude=116.670815, elevation_m=337.83)


def load_tle(tle_path):
    """Loads a tle.
    
    Opens TLE file and uses skyfield to load each pair of TLE lines
    into an EarthSatellite object. Determines epoch for each.

    Agrs:
        the_path: Path to tle file

    Returns:
        sats: List of skyfield earth satellite objects
        epochs: Time at which each set of TLE lines is most accurate
    """

    # open a Two Line Element (TLE) file
    with open(tle_path, "r") as f:
        tle_list = [line.strip() for line in f.read().split("\n") if line is not ""]

    # List of satellite ephemeris from each set of TLE in the opened file
    # Epoch: Time at which TLE is most accurate (in JD - Julian Date)
    sats = []
    epochs = []

    for i in range(0, len(tle_list), 2):
        sat = sf.sgp4lib.EarthSatellite(tle_list[i], tle_list[i + 1])
        epoch = sat.model.jdsatepoch
        sats.append(sat)
        epochs.append(epoch)

    sats = np.asarray(sats)
    epochs = np.asarray(epochs)

    return (sats, epochs)


def epoch_ranges(epochs):
    """Creates a time array
    
    Time array with intervals corresponding to epochs of 
    best accuracy from the TLE pair of lines in TLE file.
    This is done to ensure that the most relevanrt TLE is
    used in the analysis.
     
     Args:
        epochs: List of epochs derived from TLE file

     Returns:
        epoch_range: List of times, between which TLE is most accurate
    """

    # Midpoints between epochs of successive TLEs
    midpoints = list((epochs[1:] + epochs[:-1]) / 2)
    epoch_range = [epochs[0]] + midpoints + [epochs[-1]]

    return epoch_range


def epoch_time_array(epoch_range, index_epoch, cadence):
    """Create a time array.
    
    Skyfield time object [time vector] at which
    the sat position will be determined.

    Args:
        index_epoch: Index of epoch range to be converted to time array
        t_step: Cadence of time array
        epoch_range: List of times, between which TLE is most accurate

    Returns:
        t_arr: Skyfield time object
        index_epoch: Same as above
    """

    # find time between epochs/ midpoints in seconds, using Astropy Time
    t1 = Time(epoch_range[index_epoch], scale="tt", format="jd").gps
    t2 = Time(epoch_range[index_epoch + 1], scale="tt", format="jd").gps
    dt = round(t2 - t1)

    t3 = Time(epoch_range[index_epoch], scale="tt", format="jd").iso
    date, time = t3.split()
    year, month, day = date.split("-")
    hour, minute, _ = time.split(":")

    # Create a time array, every 2[cadence] seconds, starting at the first epoch, approximately
    seconds = np.arange(0, dt, cadence)
    t_arr = ts.tt(int(year), int(month), int(day), int(hour), int(minute), seconds)

    return (t_arr, index_epoch)


def sat_pass(sats, t_arr, index_epoch):
    """Finds satellite passes.
    
    Calculates alt/az of sat at every position instant of time 
    array (t_arr). Finds all the times that the sat is above
    the horizon, and returns the index t_arr at which the sat
    rose and set.
    
    Args:
        sats: list of Skyfield EarthSatellite objects
        t_arr: Skyfield time object
        index_epoch: Position in epoch_range
    
    Returns:
        passes: 2D array with pairs of indicies corresponding to rise/set of sat
        alt: Array of altitudes of sat at t_arr times
        az: Array of azimuths of sat at t_arr times
    """

    # Define Satellite to be the first one in sats list
    # Find possition of sat at each timestep of time array

    if len(t_arr) > 0:

        satellite = sats[index_epoch]
        orbit = (satellite - MWA).at(t_arr)
        alt, az, _ = orbit.altaz()

        # Check if sat is above the horizon (above -1 degrees), return boolean array
        above_horizon = alt.degrees >= -1

        # Indicies of rare times that sats are above the horizon
        (indicies,) = above_horizon.nonzero()

        # Boundary times at which the sat either rises or sets
        (boundaries,) = np.diff(above_horizon).nonzero()

        if above_horizon[0] == True:
            boundaries = [indicies[0]] + list(boundaries)
            boundaries = np.asarray(boundaries)

        if above_horizon[-1] == True:
            boundaries = list(boundaries) + [indicies[-1]]
            boundaries = np.asarray(boundaries)

        if above_horizon[-1] == True and above_horizon[0] == True:
            boundaries = [indicies[0]] + list(boundaries) + [indicies[-1]]
            boundaries = np.asarray(boundaries)

        # Reshape into pairs rise & set indicies
        passes = boundaries.reshape(len(boundaries) // 2, 2)

        return (passes, alt, az)


def ephem_data(t_arr, pass_index, alt, az):
    """Satellite Ephemeris Data.
    
    creates rise time, set times and alt, az arrays.
    
    Args:
        t_arr: Skyfield time array object
        pass_index: One pair of sat indicies from passed 2D array
        alt: Array of altitudes of sat at t_arr times
        az: Array of azimuths of sat at t_arr times

    Returns:
        t_rise: Rise time of sat in gps seconds (Astropy)
        t_set: Set time of sat in gps seconds (Astropy)
        sat_alt: Altitude of satellite while it is above the horizon. Array.
        sat_az: Azimuth of satellite while it is above the horizon. Array.
    """

    i, j = pass_index

    # A list of times at which alt/az were calculated
    # Convert to unix time to match the rf explorer timestamps
    time_array = Time(t_arr[i : j + 1].tt, scale="tt", format="jd").unix

    theta = list(az.radians)
    r = list(alt.degrees)

    sat_az = theta[i : j + 1]
    sat_alt = r[i : j + 1]

    return (time_array, sat_alt, sat_az)


def sat_plot(sat_id, alt, az, num_passes, alpha=0.3):
    """Plots satellite passes
    
    Args:
        alt: list of altitude values
        az: list of azimuth values
        num_passes: Number of satellite passes
    """

    import matplotlib

    # Force matplotlib to not use X-Server backend
    matplotlib.use("Agg")
    import matplotlib.pyplot as plt

    # Set up the polar plot.
    plt.style.use("dark_background")
    figure = plt.figure(figsize=(6, 6))
    ax = figure.add_subplot(111, polar=True)
    ax.set_ylim(90, 0)
    ax.set_rgrids([0, 30, 60, 90], angle=22)
    ax.set_theta_zero_location("N")
    ax.set_theta_direction(-1)
    ax.set_title(f"Satellite {sat_id} Sky Coverage: {num_passes} Passes", y=1.08)
    ax.grid(color="#8bbabb", linewidth=1.6, alpha=0.6)
    plt.tight_layout()

    for i in range(len(alt)):
        plt.plot(az[i], alt[i], "-", linewidth=1.6, alpha=alpha, color="#1f4e5f")

    # Return plot for saving, showing, etc
    return plt


if __name__ == "__main__":

    import os
    import json
    import argparse

    parser = argparse.ArgumentParser(
        description="""
            Code which converts the TLE files downloaded with download_TLE.py
            into satellite ephemeris data: rise time, set time, alt/az arrays
            at a given time cadence. This is saved to a json file which will 
            be used to plot the satellite passes.
            """
    )

    parser.add_argument(
        "--sat", metavar="\b", help="The Norad cat ID of satellite. Example: 21576"
    )
    parser.add_argument(
        "--tle_dir",
        metavar="\b",
        default="./../../outputs/sat_ephemeris/TLE",
        help="Directory where TLE files are saved. Default=./../../outputs/sat_ephemeris/TLE",
    )
    parser.add_argument(
        "--cadence",
        metavar="\b",
        default=4,
        help="Rate at which sat alt/az is computed. Default=4s",
    )
    parser.add_argument(
        "--out_dir",
        metavar="\b",
        default="./../../outputs/sat_ephemeris/ephem_json/",
        help="Path to output directory. Default=./../../outputs/sat_ephemeris/ephem_json/",
    )

    args = parser.parse_args()
    sat_name = args.sat
    tle_dir = args.tle_dir
    cadence = int(args.cadence)
    out_dir = args.out_dir

    tle_path = f"{tle_dir}/{sat_name}.txt"

    os.makedirs(os.path.dirname(out_dir), exist_ok=True)

    sat_ephem = {}
    sat_ephem["sat_id"] = [sat_name]
    sat_ephem["time_array"] = []
    sat_ephem["sat_alt"] = []
    sat_ephem["sat_az"] = []

    sats, epochs = load_tle(tle_path)
    epoch_range = epoch_ranges(epochs)

    for i in range(len(epoch_range) - 1):
        try:
            t_arr, index_epoch = epoch_time_array(epoch_range, i, cadence)
            passes, alt, az = sat_pass(sats, t_arr, index_epoch)

            for pass_index in passes:
                time_array, sat_alt, sat_az = ephem_data(t_arr, pass_index, alt, az)

                time_array = list(time_array)

                # We don't care about sat passes shorter than a minute (3*20sec)
                if len(time_array) >= 3:
                    sat_ephem["time_array"].append(time_array)
                    sat_ephem["sat_alt"].append(sat_alt)
                    sat_ephem["sat_az"].append(sat_az)
        except Exception:
            pass

    with open(f"{out_dir}/{sat_name}.json", "w") as outfile:
        json.dump(sat_ephem, outfile, indent=4)



# Force matplotlib to not use X-Server backend
import matplotlib

matplotlib.use("Agg")
import matplotlib.pyplot as plt
import matplotlib.pylab as pl
import seaborn as sns
import numpy as np


def plt_waterfall_pass(
    power, sat_id, start, stop, date, cmap, chs=None, good_ch=None, out_dir=None
):
    """Plot waterfall with sat window and occupied channels
    
    Args:
        power:          RF power array
        sat_id:         Norad cat ID
        start:          Start of epehm for sat_id
        stop:           Stop of ephem of sat_id
        chs:            Occupied channels [list]
        good_ch:        Most probable channel
        date:           Date of observation
    """

    plt.style.use("dark_background")
    fig = plt.figure(figsize=(7, 10))
    ax = fig.add_axes([0.12, 0.1, 0.72, 0.85])
    im = ax.imshow(power, interpolation="none", cmap=cmap)
    cax = fig.add_axes([0.88, 0.1, 0.03, 0.85])
    fig.colorbar(im, cax=cax)
    ax.set_aspect("auto")
    ax.set_title(f"Waterfall Plot: {sat_id} in {chs}")

    ax.set_xlabel("Freq Channel")
    ax.set_ylabel("Time Step")

    # horizontal highlight of ephem
    ax.axhspan(start, stop, alpha=0.1, color="white")

    if chs != None:
        # vertical highlight of channel
        for ch in chs:
            ax.axvspan(ch - 1.0, ch + 0.6, alpha=0.2, color="white")

    if good_ch != None:
        # highlight good channel
        ax.axvspan(
            good_ch - 1.0,
            good_ch + 0.6,
            alpha=0.6,
            edgecolor="#a7ff83",
            facecolor=None,
            fill=False,
        )

    # plt.show()
    if chs != None:
        plt.savefig(f"{out_dir}/{date}_{sat_id}_{chs}_waterfall.png")
    else:
        plt.savefig(f"{out_dir}/{date}_{sat_id}_waterfall.png")

    plt.close()
    plt.rcParams.update(plt.rcParamsDefault)


def plt_channel_basic(
    out_dir,
    times,
    channel_power,
    chan_num,
    min_s,
    max_s,
    noise_threshold,
    arbitrary_threshold,
    sat_id,
    date,
):

    """Plot power in channel, with various thresholds
    
    Args:
        times:          Time array
        channel_power:  Power in channel
        chan_num:       Channel Number
        min_s:          Minimum signal in channel_power
        max_s:          Maximum signal in channel_power
        noise_threshold: Noise Threshold (n*MAD)
        arbitrary_threshold: Arbitrary threshold used to only select bright passes
        sat_id:         Norad Cat ID
        date:           Date of observation
        """

    plt.style.use("seaborn")

    # plt channel power
    plt.plot(
        times,
        channel_power,
        linestyle="-",
        linewidth=2,
        alpha=1.0,
        color="#db3751",
        label="Data",
    )
    plt.fill_between(times, channel_power, color="#db3751", alpha=0.7)

    plt.axhline(
        arbitrary_threshold,
        alpha=1.0,
        linestyle="-",
        linewidth=2,
        color="#fba95f",
        label=f"Arbitrary Cut: {arbitrary_threshold} dBm",
    )
    plt.axhspan(-1, arbitrary_threshold, color="#fba95f", alpha=0.4)

    plt.axhline(
        noise_threshold,
        linestyle="-",
        linewidth=2,
        color="#5cb7a9",
        label=f"Noise Cut: {noise_threshold:.2f} dBm",
    )
    plt.axhspan(-1, noise_threshold, color="#5cb7a9", alpha=0.4)

    plt.ylim([min_s - 1, max_s + 1])
    plt.xlim([times[0], times[-1]])
    plt.ylabel("Power [dBm]")
    plt.xlabel("Time [s]")
    plt.title(f"Satellite Pass in Channel: [{chan_num}]")
    plt.tight_layout()
    leg = plt.legend(frameon=True)
    leg.get_frame().set_facecolor("grey")
    leg.get_frame().set_alpha(0.2)
    for l in leg.legendHandles:
        l.set_alpha(1)
    plt.savefig(f"{out_dir}/{date}_{sat_id}_{chan_num}_channel.png")
    plt.close()
    plt.rcParams.update(plt.rcParamsDefault)


def sat_plot(out_dir, ids, norad_id, alt, az, num_passes, date, name):
    """Plots satellite passes
    
    Args:
        alt: list of altitude values
        az: list of azimuth values
        num_passes: Number of satellite passes
    """

    # Set up the polar plot.
    plt.style.use("seaborn")
    plt.style.use("dark_background")
    figure = plt.figure(figsize=(7, 6))
    ax = figure.add_subplot(111, polar=True)
    ax.set_ylim(90, 0)
    ax.set_rgrids([0, 30, 60, 90], angle=22)
    ax.set_theta_zero_location("N")
    ax.set_theta_direction(-1)
    ax.set_title(f"{num_passes} satellite passes in {date} [{norad_id}] window", y=1.05)
    ax.grid(color="grey", linewidth=1.6, alpha=0.5)

    colors = pl.cm.Spectral(np.linspace(0.17, 0.9, len(alt)))

    for i in range(len(alt)):
        plt.plot(
            az[i],
            alt[i],
            "-",
            linewidth=2.4,
            alpha=0.8,
            color=colors[i],
            label=f"{ids[i]}",
        )
        plt.legend()

    leg = plt.legend(
        frameon=True,
        bbox_to_anchor=(0.28, 1.0, 1.0, -0.95),
        loc="center right",
        title="Norad SatID",
    )
    leg.get_frame().set_facecolor("grey")
    leg.get_frame().set_alpha(0.4)
    for l in leg.legendHandles:
        l.set_alpha(1)

    plt.tight_layout()
    plt.savefig(f"{out_dir}/{date}_{norad_id}_{name}.png")
    plt.close()
    plt.rcParams.update(plt.rcParamsDefault)



import sys
import numpy as np
import healpy as hp
import scipy.optimize as opt
import numpy.polynomial.polynomial as poly
from mwa_pb.mwa_sweet_spots import all_grid_points

sys.path.append("../decode_rf_data")
from colormap import spectral, jade, kelp

# Custom spectral colormap
jade, _ = jade()


def hp_slices_horizon(nside=None, zenith_angle=90):
    """Healpix pix indices of NS, EW slices and above horizon"""
    # theta phi values of each pixel
    hp_indices = np.arange(hp.nside2npix(nside))
    θ, ɸ = hp.pix2ang(nside, hp_indices)

    # healpix indices above the horizon
    above_horizon_indices = np.where(θ <= np.radians(zenith_angle))[0]

    # pixel coords above the horizon
    ɸ_above_horizon = ɸ[above_horizon_indices]

    NS_indices = []
    EW_indices = []

    # pixel indices along N, E, S, W slices
    # order the indices such that they proceed from N -> S or E -> W
    n_slice = sorted(np.where((np.round(np.degrees(ɸ_above_horizon))) == 45)[0])
    e_slice = sorted(np.where((np.round(np.degrees(ɸ_above_horizon))) == 135)[0])
    s_slice = sorted(
        np.where((np.round(np.degrees(ɸ_above_horizon))) == 225)[0], reverse=True
    )
    w_slice = sorted(
        np.where((np.round(np.degrees(ɸ_above_horizon))) == 315)[0], reverse=True
    )

    NS_indices.extend(s_slice)
    NS_indices.extend(n_slice)
    EW_indices.extend(w_slice)
    EW_indices.extend(e_slice)

    return [NS_indices, EW_indices, above_horizon_indices]


def slice_map(hp_map, za):
    """slices healpix map along NS, EW"""

    NS_indices, EW_indices, _ = hp_slices_horizon(nside, zenith_angle=za)

    θ_NS, ɸ_NS = np.degrees(hp.pix2ang(nside, NS_indices))
    θ_EW, ɸ_EW = np.degrees(hp.pix2ang(nside, EW_indices))

    zenith_angle_NS = []
    for i, j in zip(θ_NS, ɸ_NS):
        if j <= 180:
            zenith_angle_NS.append(-1 * i)
        else:
            zenith_angle_NS.append(i)

    zenith_angle_EW = []
    for i, j in zip(θ_EW, ɸ_EW):
        if j <= 180:
            zenith_angle_EW.append(-1 * i)
        else:
            zenith_angle_EW.append(i)

    NS_data = [hp_map[NS_indices], zenith_angle_NS]
    EW_data = [hp_map[EW_indices], zenith_angle_EW]

    return [NS_data, EW_data]


def nan_mad(ref_map):
    """Compute mad while ignoring nans"""
    ref_map_mad = []
    for j in ref_map:
        if j != []:
            j = np.asarray(j)
            j = j[~np.isnan(j)]
            ref_map_mad.append(mad(j))
        else:
            ref_map_mad.append(np.nan)

    ref_map_mad = np.asarray(ref_map_mad)
    ref_map_mad[np.where(ref_map_mad == np.nan)] = np.nanmean(ref_map_mad)

    return ref_map_mad


def tile_map_slice(good_map, za):
    """slices ref healpix map along NS & EW"""

    ref_map_NS, ref_map_EW = slice_map(np.asarray(good_map), za)

    ref_med_map_NS = np.asarray(
        [(np.nanmedian(i) if i != [] else np.nan) for i in ref_map_NS[0]]
    )
    ref_mad_map_NS = np.asarray(nan_mad(ref_map_NS[0]))
    za_NS = ref_map_NS[1]

    ref_med_map_EW = np.asarray(
        [(np.nanmedian(i) if i != [] else np.nan) for i in ref_map_EW[0]]
    )
    ref_mad_map_EW = np.asarray(nan_mad(ref_map_EW[0]))
    za_EW = ref_map_EW[1]

    NS_data = [ref_med_map_NS, ref_mad_map_NS, za_NS]
    EW_data = [ref_med_map_EW, ref_mad_map_EW, za_EW]

    return [NS_data, EW_data]


# rotate func written by Jack Line
def rotate(nside, angle=None, healpix_array=None, savetag=None, flip=False):
    """Takes in a healpix array, rotates it by the desired angle, and saves it.
    Optionally flip the data, changes east-west into west-east because
    astronomy"""

    # theta phi values of each pixel
    hp_indices = np.arange(hp.nside2npix(nside))
    θ, ɸ = hp.pix2ang(nside, hp_indices)

    new_hp_inds = hp.ang2pix(nside, θ, ɸ + angle)

    ##Flip the data to match astro conventions
    if flip == True:
        new_angles = []
        for phi in ɸ:
            if phi <= np.pi:
                new_angles.append(np.pi - phi)
            else:
                new_angles.append(3 * np.pi - phi)
        new_hp_inds = hp.ang2pix(nside, ɸ, np.asarray(new_angles))

    ##Save the array in the new order
    if savetag:
        np.savez_compressed(savetag, beammap=healpix_array[new_hp_inds])

    return healpix_array[new_hp_inds]


# chisquared minimization to best fit map to data
def fit_gain(map_data=None, map_error=None, beam=None):
    """Fit the beam model to the measured data using
    chisquared minimization"""

    bad_values = np.isnan(map_data)
    map_data = map_data[~bad_values]
    map_error = map_error[~bad_values]

    map_error[np.where(map_error == 0)] = np.mean(map_error)

    def chisqfunc(gain):
        model = beam[~bad_values] + gain
        chisq = sum((map_data - model) ** 2)
        # chisq = sum(((map_data - model)/map_error)**2)
        return chisq

    x0 = np.array([0])

    result = opt.minimize(chisqfunc, x0)

    return result.x


def poly_fit(x, y, map_data, order):
    """Fit polynominal of order to data"""

    x = np.asarray(x)
    y = np.asarray(y)

    bad_values = np.isnan(map_data)
    x_good = x[~bad_values]
    y_good = y[~bad_values]
    coefs = poly.polyfit(x_good, y_good, order)
    fit = poly.polyval(x, coefs)
    return fit


def plt_slice(
    fig=None,
    sub=None,
    zen_angle=None,
    map_slice=None,
    map_error=None,
    model_slice=None,
    delta_pow=None,
    pow_fit=None,
    slice_label=None,
    model_label=None,
):

    """Plot a slice of the beam, with measured
    data, errorbars, and fit the simulated beam
    to the data. Also plot the diff b/w data and
    the model"""

    ax = fig.add_subplot(sub)

    ax.errorbar(
        zen_angle,
        map_slice,
        yerr=map_error,
        fmt=".",
        color="#326765",
        ecolor="#7da87b",
        elinewidth=1.2,
        capsize=1.2,
        capthick=1.4,
        alpha=0.9,
        label=slice_label,
    )

    ax.plot(
        zen_angle,
        model_slice,
        color="#c70039",
        linewidth=1.2,
        alpha=0.9,
        label=model_label,
    )

    ax.set_ylabel("Power (dB)")
    # ax.set_ylim(bottom=-30)
    ax.set_xlabel("Zenith Angle (degrees)")
    ax.legend()
    ax.set_xlim([-92, 92])
    ax.set_ylim([-50, 5])

    divider = make_axes_locatable(ax)
    dax = divider.append_axes("bottom", size="30%", pad=0.1)

    # dax = fig.add_subplot(2,1,2)
    dax.scatter(zen_angle, delta_pow, marker=".", color="#27296d")
    dax.plot(zen_angle, pow_fit, linewidth=1.2, alpha=0.9, color="#ff8264")
    dax.set_ylabel("Data - Model (dB)")
    dax.set_ylim([-10, 10])
    # dax.set_xticklabels([])
    dax.set_xlim([-92, 92])

    return ax


def plot_healpix(
    data_map=None,
    fig=None,
    sub=None,
    title=None,
    vmin=None,
    vmax=None,
    cmap=None,
    cbar=True,
):
    """Yeesh do some healpix magic to plot the thing"""

    # Disable cryptic healpy warnings. Can't figure out where they originate
    import warnings

    warnings.filterwarnings("ignore", category=RuntimeWarning)

    hp.orthview(
        map=data_map,
        coord="E",
        fig=fig,
        half_sky=True,
        rot=(0, 90, 180),
        xsize=600,
        title=title,
        sub=sub,
        min=vmin,
        max=vmax,
        cmap=cmap,
        notext=True,
        hold=True,
        cbar=cbar,
    )

    hp.graticule(dpar=10, coord="E", color="k", alpha=0.3, dmer=45)

    # Altitude grid
    hp.projtext(
        00.0 * (np.pi / 180.0), 225.0 * (np.pi / 180), "0", color="w", coord="E"
    )
    hp.projtext(
        30.0 * (np.pi / 180.0), 225.0 * (np.pi / 180), "30", color="w", coord="E"
    )
    hp.projtext(
        60.0 * (np.pi / 180.0), 225.0 * (np.pi / 180), "60", color="w", coord="E"
    )

    # NSEW
    hp.projtext(
        80.0 * (np.pi / 180.0),
        000.0 * (np.pi / 180.0),
        r"$N  $",
        coord="E",
        color="w",
        verticalalignment="top",
    )
    hp.projtext(
        80.0 * (np.pi / 180.0),
        090.0 * (np.pi / 180.0),
        r"$E  $",
        coord="E",
        color="w",
        horizontalalignment="right",
    )
    hp.projtext(
        80.0 * (np.pi / 180.0),
        180.0 * (np.pi / 180.0),
        r"$S  $",
        coord="E",
        color="w",
        verticalalignment="bottom",
    )
    hp.projtext(
        80.0 * (np.pi / 180.0),
        270.0 * (np.pi / 180.0),
        r"$W  $",
        coord="E",
        color="w",
        horizontalalignment="left",
    )


def beam_slice(f):

    f_name, _ = f.name.split(".")
    t_name, r_name, _, _ = f_name.split("_")

    pointings = ["0", "2", "4", "41"]

    # load data from map .npz file
    tile_map = np.load(f, allow_pickle=True)
    fee_m = np.load(fee_map, allow_pickle=True)

    for p in pointings:

        tile = tile_map[p]

        if "XX" in t_name:
            fee = fee_m[p][0]
        else:
            fee = fee_m[p][1]

        # rotate maps so slices can be taken
        fee_r = rotate(nside, angle=-np.pi / 4, healpix_array=fee)
        tile_r = rotate(nside, angle=-np.pi / 4, healpix_array=tile)

        # slice the tile and fee maps along NS, EW
        # zenith angle thresh of 70 to determine fit gain factor
        NS_f, EW_f = slice_map(fee_r, 70)
        NS_t, EW_t = tile_map_slice(tile_r, 70)

        gain_NS = fit_gain(map_data=NS_t[0], map_error=NS_t[1], beam=NS_f[0])
        gain_EW = fit_gain(map_data=EW_t[0], map_error=EW_t[1], beam=EW_f[0])

        # slice the tile and fee maps along NS, EW.
        # the above gain factor is applied to full beam slices
        NS_fee, EW_fee = slice_map(fee_r, 90)
        NS_tile, EW_tile = tile_map_slice(tile_r, 90)

        # Scale the data so that it best fits the beam slice
        NS_tile_med = NS_tile[0] - gain_NS[0]
        EW_tile_med = EW_tile[0] - gain_EW[0]

        # delta powers
        del_NS = NS_tile_med - NS_fee[0]
        del_EW = EW_tile_med - EW_fee[0]

        # 3rd order poly fits for residuals
        fit_NS = poly_fit(NS_tile[2], del_NS, NS_tile[0], 3)
        fit_EW = poly_fit(EW_tile[2], del_EW, EW_tile[0], 3)

        # Visualize the tile map and diff map
        # healpix meadian map
        tile_med = np.asarray([(np.nanmedian(j) if j != [] else np.nan) for j in tile])

        residuals = tile_med - fee
        residuals[np.where(fee < -30)] = np.nan
        residuals[np.where(tile_med == np.nan)] = np.nan

        # This is an Awesome plot
        plt.style.use("seaborn")
        fig1 = plt.figure(figsize=(10, 8))

        ax1 = plt.subplot(2, 2, 1)
        plt_slice(
            fig=fig1,
            sub=221,
            zen_angle=NS_tile[2],
            map_slice=NS_tile_med,
            map_error=NS_tile[1],
            model_slice=NS_fee[0],
            delta_pow=del_NS,
            pow_fit=fit_NS,
            slice_label="Tile NS",
            model_label="FEE NS",
        )

        ax2 = fig1.add_axes([0.48, 0.52, 0.48, 0.43])
        plot_healpix(
            data_map=tile_med,
            sub=(2, 2, 2),
            fig=fig1,
            title="tile map",
            cmap=jade,
            vmin=-50,
            vmax=0,
            cbar=False,
        )
        ax7 = plt.gca()
        image = ax7.get_images()[0]
        cax = fig1.add_axes([0.92, 0.52, 0.015, 0.43])
        cbar = fig1.colorbar(image, cax=cax, label="dB")

        ax3 = plt.subplot(2, 2, 3)
        plt_slice(
            fig=fig1,
            sub=223,
            zen_angle=EW_tile[2],
            map_slice=EW_tile_med,
            map_error=EW_tile[1],
            model_slice=EW_fee[0],
            delta_pow=del_EW,
            pow_fit=fit_EW,
            slice_label="Tile EW",
            model_label="FEE EW",
        )

        ax4 = fig1.add_axes([0.48, 0.02, 0.48, 0.43])
        plot_healpix(
            data_map=residuals,
            sub=(2, 2, 4),
            fig=fig1,
            title="diff map",
            cmap="inferno",
            vmin=-10,
            vmax=5,
            cbar=False,
        )
        ax8 = plt.gca()
        image = ax8.get_images()[0]
        cax = fig1.add_axes([0.92, 0.02, 0.015, 0.43])
        cbar = fig1.colorbar(image, cax=cax, label="dB")

        plt.tight_layout()
        plt.savefig(f"{out_dir}/{p}/{t_name}_{r_name}_{p}_beam_slices.png")
        plt.close()


if __name__ == "__main__":

    import argparse
    import numpy as np
    from pathlib import Path
    import concurrent.futures
    import matplotlib

    matplotlib.use("Agg")
    import matplotlib.pyplot as plt
    import matplotlib.gridspec as gs
    from mpl_toolkits.axes_grid1 import make_axes_locatable
    from scipy.stats import median_absolute_deviation as mad

    import sys

    sys.path.append("../decode_rf_data")
    from colormap import spectral

    # Custom spectral colormap
    cmap = spectral()

    parser = argparse.ArgumentParser(
        description="""
        Plot healpix map of reference data
        """
    )

    parser.add_argument(
        "--out_dir",
        metavar="\b",
        default="../../outputs/tile_maps/compare_beams/",
        help="Output directory. Default=../../outputs/tile_maps/compare_beams/",
    )
    parser.add_argument(
        "--map_dir",
        metavar="\b",
        default="../../outputs/tile_maps/tile_maps_norm/",
        help="Tile map directory. Default=../../outputs/tile_maps/tile_maps_norm/",
    )
    parser.add_argument(
        "--fee_map",
        metavar="\b",
        default="../../outputs/tile_maps/FEE_maps/mwa_fee_beam.npz",
        help="Healpix FEE map of mwa tile. default=../../outputs/tile_maps/FEE_maps/mwa_fee_beam.npz",
    )
    parser.add_argument(
        "--nside",
        metavar="\b",
        type=int,
        default=32,
        help="Healpix Nside. Default = 32",
    )

    args = parser.parse_args()

    out_dir = Path(args.out_dir)
    map_dir = Path(args.map_dir)
    fee_map = args.fee_map
    nside = args.nside

    # make output dir if it doesn't exist
    out_dir.mkdir(parents=True, exist_ok=True)

    # MWA beam pointings
    pointings = ["0", "2", "4", "41"]

    # Create output directory structure
    for p in pointings:
        Path(f"{out_dir}/{p}/").mkdir(parents=True, exist_ok=True)

    # find all map files
    map_files = [item for item in map_dir.glob("*.npz")]

    # Parallization magic happens here
    with concurrent.futures.ProcessPoolExecutor() as executor:
        results = executor.map(beam_slice, map_files)
#    beam_slice(map_files[0])



import sys
import numpy as np
import healpy as hp
from scipy.stats import median_absolute_deviation as mad

sys.path.append("../sat_ephemeris")
from sat_ids import norad_ids

from plot_tile_maps import plot_healpix


def sat_maps(sat):

    f = Path(f"{map_dir}/S07XX_rf0XX_sat_maps.npz")
    f_name, _ = f.name.split(".")
    tile, ref, _, _ = f_name.split("_")

    pointings = ["0", "2", "4", "41"]

    # list of all possible satellites
    sat_ids = list(norad_ids.values())

    # load data from map .npz file
    tile_data = np.load(f, allow_pickle=True)
    tile_data = {key: tile_data[key].item() for key in tile_data}
    tile_map = tile_data["mwa_map"]

    for p in pointings:

        fig = plt.figure(figsize=(8, 10))
        fig.suptitle(f"Satellite [{sat}]: {tile}/{ref} @ {p}", fontsize=16)
        tile_sat_med = [
            (np.median(i) if i != [] else np.nan) for i in tile_data["mwa_map"][p][sat]
        ]
        # tile_sat_scaled = np.asarray([(i - np.nanmax(tile_sat_med[:5000])) for i in tile_sat_med])
        plot_healpix(data_map=np.asarray(tile_sat_med), sub=(1, 1, 1), cmap=jade)
        plt.savefig(
            f"{out_dir}/sat_maps/{p}/{sat}_{p}_{tile}_{ref}_passes.png",
            bbox_inches="tight",
        )
        plt.close()


if __name__ == "__main__":

    import argparse
    import matplotlib

    matplotlib.use("Agg")
    from pathlib import Path
    import concurrent.futures
    import matplotlib.pyplot as plt

    import sys

    sys.path.append("../decode_rf_data")
    from colormap import spectral, jade, kelp

    # Custom spectral colormap
    jade, _ = jade()

    parser = argparse.ArgumentParser(
        description="""
        Plot healpix map of reference data
        """
    )

    parser.add_argument(
        "--out_dir",
        metavar="\b",
        default="./../../outputs/tile_maps/",
        help="Output directory. Default=./../../outputs/tile_maps/",
    )
    parser.add_argument(
        "--map_dir",
        metavar="\b",
        default="./../../outputs/tile_maps/tile_maps_raw",
        help="Output directory. Default=./../../outputs/tile_maps/tile_maps_raw",
    )
    parser.add_argument(
        "--nside",
        metavar="\b",
        type=int,
        default=32,
        help="Healpix Nside. Default = 32",
    )

    args = parser.parse_args()

    out_dir = Path(args.out_dir)
    map_dir = Path(args.map_dir)
    nside = args.nside

    # list of all possible satellites
    sat_ids = list(norad_ids.values())

    pointings = ["0", "2", "4", "41"]

    map_files = [item for item in map_dir.glob("*.npz")]

    # Create output directory tree
    for p in pointings:
        Path(f"{out_dir}/sat_maps/{p}/").mkdir(parents=True, exist_ok=True)

    # plot maps for all sats, for only one tile_ref pair
    # S08XX has good time coverage
    with concurrent.futures.ProcessPoolExecutor() as executor:
        results = executor.map(sat_maps, sat_ids)
    # sat_maps(sat_ids[0])



import os
import json
import sat_ids
import concurrent.futures
from itertools import repeat
from sat_ephemeris import sat_plot


def plot_ephem(json_dir, out_dir, sat_id):
    try:
        with open(f"{json_dir}/{sat_id}.json", "r") as ephem:
            sat_ephem = json.load(ephem)
            altitude = sat_ephem["sat_alt"]
            azimuth = sat_ephem["sat_az"]
            num_passes = len(sat_ephem["time_array"])

            plt = sat_plot(sat_id, altitude, azimuth, num_passes)
            plt.savefig(f"{out_dir}/{sat_id}.png")
            plt.close()

    except Exception:
        print(f"Could not plot {sat_id}.png. Source json file missing ")


if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser(
        description=""" 
            Plot Satellite passes from json ephemeris files.
            """
    )

    parser.add_argument(
        "--out_dir",
        metavar="\b",
        default="./../../outputs/sat_ephemeris/ephem_plots/",
        help="Path to output directory. Default=./../../outputs/sat_ephemeris/ephem_plots/",
    )
    parser.add_argument(
        "--json_dir",
        metavar="\b",
        default="./../../outputs/sat_ephemeris/ephem_json/",
        help="Path to ephem_json directory. Default=./../../outputs/sat_ephemeris/ephem_json/",
    )

    args = parser.parse_args()

    out_dir = args.out_dir
    json_dir = args.json_dir

    # Create output directory
    os.makedirs(os.path.dirname(out_dir), exist_ok=True)

    # Load list of satellites (Nodar ids)
    sat_list = list(sat_ids.norad_ids.values())

    # Parellization Magic Here!
    with concurrent.futures.ProcessPoolExecutor() as executor:
        results = executor.map(plot_ephem, repeat(json_dir), repeat(out_dir), sat_list)



import json
import numpy as np
from pathlib import Path


def org_pointing_json(meta_dir):
    """Organize json files. Clean up quirks in data such 
    and Andrew's custom pointing names"""

    start_gps = []
    stop_gps = []
    obs_length = []
    pointings = []

    point_0 = ["All_0", "Zenith_Test"]
    point_2 = ["EOR_Point_2", "EOR_Point_2_Delays0,1,2,3,0,1,2,3,0,1,2,3,0,1,2,3_Ch100"]
    point_4 = ["EOR_Point_4", "EOR_Point_4_Delays3,2,1,0,3,2,1,0,3,2,1,0,3,2,1,0_Ch100"]

    files = meta_dir.glob("pointings*.json")

    for f in files:
        with open(f) as table:
            data = json.load(table)
            for i in range(len(data)):
                name = data[i][2]
                pointing = data[i][-1]
                start = data[i][0]

                # Many obs are named None, but their title indicates their true pointing
                # These are mostly obs which Andrew Williams schduled for this experiment
                if name in point_0:
                    pointing = 0
                elif name in point_2:
                    pointing = 2
                elif name in point_4:
                    pointing = 4
                else:
                    pass

                ## Filter obs by passes
                # if pointing in [0, 2, 4]:
                if pointing in list(range(197)):

                    if i != (len(data) - 1):
                        # Telescope pointing remains constant till it is changed
                        # So, stop time is the start of next observation
                        stop = data[i + 1][0]
                    else:
                        stop = data[i][1]

                    length = stop - start

                    if length >= 120:

                        start_gps.append(start)
                        stop_gps.append(stop)
                        obs_length.append(length)
                        pointings.append(pointing)

    return (start_gps, stop_gps, obs_length, pointings)


def combine_obs(start_gps, stop_gps, obs_length, pointings, out_dir):
    # if consecutive obs have same pointing, combine them.
    for i in range(len(start_gps)):
        if i != len(start_gps) - 1:
            if (stop_gps[i] == start_gps[i + 1]) and (pointings[i] == pointings[i + 1]):
                start_gps[i + 1] = start_gps[i]
                obs_length[i + 1] += obs_length[i]

                pointings[i] = None

    # Apply mask, couldn't think of a better way to implement this
    good_idx = np.where(np.asarray(pointings) != None)[0]
    start_gps = np.asarray(start_gps)[good_idx].tolist()
    stop_gps = np.asarray(stop_gps)[good_idx].tolist()
    obs_length = np.asarray(obs_length)[good_idx].tolist()
    pointings = np.asarray(pointings)[good_idx].tolist()

    # Create dictionary to be saved to json, for plotting
    pointing_list = {}
    pointing_list["grid_pt"] = pointings
    pointing_list["start_gps"] = start_gps
    pointing_list["stop_gps"] = stop_gps
    pointing_list["obs_length"] = obs_length

    with open(f"{out_dir}/ultimate_pointing_times.json", "w") as outfile:
        json.dump(pointing_list, outfile, indent=4)


if __name__ == "__main__":
    import argparse

    parser = argparse.ArgumentParser(
        description="""
            Reads data from downloaded pointing matadata
            json files. Extracts the start, stop time and
            grid-pointing number.
            """
    )

    parser.add_argument(
        "--meta_dir",
        metavar="\b",
        default="./../../outputs/beam_pointings/mwa_pointings/",
        help="Directory where json metadata files live. Default=./../../outputs/beam_pointings/mwa_pointings/",
    )

    parser.add_argument(
        "--out_dir",
        metavar="\b",
        default="./../../outputs/beam_pointings/",
        help="Directory where json metadata files live. Default=./../../outputs/beam_pointings/",
    )

    args = parser.parse_args()
    meta_dir = Path(args.meta_dir)
    out_dir = Path(args.out_dir)

    start_gps, stop_gps, obs_length, pointings = org_pointing_json(meta_dir)
    combine_obs(start_gps, stop_gps, obs_length, pointings, out_dir)



import os
import wget
import numpy as np
import healpy as hp
from pathlib import Path
import matplotlib.pyplot as plt
from astropy.stats import median_absolute_deviation
from itertools import cycle

from matplotlib import rcParams
from os import environ
import scipy.optimize as opt
from mpl_toolkits.axes_grid1 import make_axes_locatable

import mwa_pb

fee_dir = Path(f"{os.path.dirname(mwa_pb.__file__)}/data")
if Path(f"{fee_dir}/mwa_full_embedded_element_pattern.h5").is_file() is not True:
    print("Downloading MWA FEE model from Cerberus")
    url = "http://cerberus.mwa128t.org/mwa_full_embedded_element_pattern.h5"
    wget.download(url, f"{fee_dir}")

MWAPY_H5PATH = f"{fee_dir}/mwa_full_embedded_element_pattern.h5"
from mwa_pb import primary_beam
from mwa_pb import beam_full_EE
from mwa_pb import mwa_tile
from mwa_pb.mwa_sweet_spots import all_grid_points
from plot_tile_maps import plot_healpix


def local_beam(
    za,
    az,
    freq,
    delays=None,
    zenithnorm=True,
    power=True,
    jones=False,
    interp=True,
    pixels_per_deg=5,
    amps=None,
):
    """Code pulled from mwapy that generates the MWA beam response - removes unecessary extra code from mwapy/pb
        - delays is a 2x16 array, with the first 16 delays for the XX, 
          second 16 for the YY pol. values match what you find in the metafits file
        - amps are the amplitudes of each individual dipole, again in a 2x16,
            with XX first then YY"""

    tile = beam_full_EE.ApertureArray(MWAPY_H5PATH, freq)
    mybeam = beam_full_EE.Beam(tile, delays, amps)
    if interp:
        j = mybeam.get_interp_response(az, za, pixels_per_deg)
    else:
        j = mybeam.get_response(az, za)
    if zenithnorm == True:
        j = tile.apply_zenith_norm_Jones(j)  # Normalise

    # Use swapaxis to place jones matrices in last 2 dimensions
    # insead of first 2 dims.
    if len(j.shape) == 4:
        j = np.swapaxes(np.swapaxes(j, 0, 2), 1, 3)
    elif len(j.shape) == 3:  # 1-D
        j = np.swapaxes(np.swapaxes(j, 1, 2), 0, 1)
    else:  # single value
        pass

    if jones:
        return j

    # Use mwa_tile makeUnpolInstrumentalResponse because we have swapped axes
    vis = mwa_tile.makeUnpolInstrumentalResponse(j, j)
    if not power:
        return (np.sqrt(vis[:, :, 0, 0].real), np.sqrt(vis[:, :, 1, 1].real))
    else:
        return (vis[:, :, 0, 0].real, vis[:, :, 1, 1].real)


if __name__ == "__main__":

    import sys
    import argparse
    from pathlib import Path

    sys.path.append("../decode_rf_data")
    from colormap import jade
    import matplotlib.pyplot as plt

    # Custom spectral colormap
    jade, _ = jade()

    parser = argparse.ArgumentParser(
        description="""
        Create Simulated MWA Beam response maps using mwapy
        """
    )

    parser.add_argument(
        "--out_dir",
        metavar="\b",
        default="./../../outputs/tile_maps/FEE_maps/",
        help="Output directory. Default=./../../outputs/tile_maps/FEE_maps/",
    )
    parser.add_argument(
        "--nside",
        metavar="\b",
        type=int,
        default=32,
        help="Healpix Nside. Default = 32",
    )

    args = parser.parse_args()

    out_dir = Path(args.out_dir)
    nside = args.nside

    # make output directory if it doesn't exist
    Path(out_dir).mkdir(parents=True, exist_ok=True)

    pointings = ["0", "2", "4", "41"]

    fee_beam = {}
    for p in pointings:

        # Empty array for model beam
        npix = hp.nside2npix(nside)
        beam_response_XX = np.zeros(npix)
        beam_response_YY = np.zeros(npix)

        # healpix indices above horizon
        # convert to zenith angle and azimuth
        above_horizon = range(int(npix / 2))
        beam_zas, beam_azs = hp.pix2ang(nside, above_horizon)

        delay_p = np.array([all_grid_points[int(p)][-1], all_grid_points[int(p)][-1]])

        # S21 had a missing dipole, so need a different amplitude array for the model
        amps = np.ones((2, 16))
        # if AUT == 'S21XX':
        #    amps[0,5] = 0

        # Make beam response
        response = local_beam(
            [list(beam_zas)],
            [list(beam_azs)],
            freq=137e6,
            delays=delay_p,
            zenithnorm=True,
            power=True,
            interp=False,
            amps=amps,
        )
        response_XX = response[0][0]
        response_YY = response[1][0]

        # Stick in an array, convert to decibels, and noralise
        beam_response_XX[above_horizon] = response_XX
        decibel_beam_XX = 10 * np.log10(beam_response_XX)
        normed_beam_XX = decibel_beam_XX - decibel_beam_XX.max()

        beam_response_YY[above_horizon] = response_YY
        decibel_beam_YY = 10 * np.log10(beam_response_YY)
        normed_beam_YY = decibel_beam_YY - decibel_beam_YY.max()

        fee_beam[p] = [normed_beam_XX, normed_beam_YY]

        fig = plt.figure(figsize=(9, 10))
        fig.suptitle(f"MWA FEE MAP @ pointing [{p}] XX", fontsize=16, y=1.0)
        plot_healpix(
            data_map=normed_beam_XX, sub=(1, 1, 1), cmap=jade, vmin=-50, vmax=0
        )
        plt.savefig(f"{out_dir}/mwa_fee_beam_{p}_XX.png")
        plt.close()

        fig = plt.figure(figsize=(9, 10))
        fig.suptitle(f"MWA FEE MAP @ pointing [{p}] YY", fontsize=16, y=1.0)
        plot_healpix(
            data_map=normed_beam_YY, sub=(1, 1, 1), cmap=jade, vmin=-50, vmax=0
        )
        plt.savefig(f"{out_dir}/mwa_fee_beam_{p}_YY.png")
        plt.close()

    np.savez_compressed(f"{out_dir}/mwa_fee_beam.npz", **fee_beam)

    fee_beam_flagged = {}
    for p in pointings:

        # Empty array for model beam
        npix = hp.nside2npix(nside)
        beam_response_XX = np.zeros(npix)
        beam_response_YY = np.zeros(npix)

        # healpix indices above horizon
        # convert to zenith angle and azimuth
        above_horizon = range(int(npix / 2))
        beam_zas, beam_azs = hp.pix2ang(nside, above_horizon)

        delay_p = np.array([all_grid_points[int(p)][-1], all_grid_points[int(p)][-1]])

        # S21 had a missing dipole, so need a different amplitude array for the model
        amps = np.ones((2, 16))
        amps[1, 8] = 0

        # Make beam response
        response = local_beam(
            [list(beam_zas)],
            [list(beam_azs)],
            freq=137e6,
            delays=delay_p,
            zenithnorm=True,
            power=True,
            interp=False,
            amps=amps,
        )
        response_XX = response[0][0]
        response_YY = response[1][0]

        # Stick in an array, convert to decibels, and noralise
        beam_response_XX[above_horizon] = response_XX
        decibel_beam_XX = 10 * np.log10(beam_response_XX)
        normed_beam_XX = decibel_beam_XX - decibel_beam_XX.max()

        beam_response_YY[above_horizon] = response_YY
        decibel_beam_YY = 10 * np.log10(beam_response_YY)
        normed_beam_YY = decibel_beam_YY - decibel_beam_YY.max()

        fee_beam_flagged[p] = [normed_beam_XX, normed_beam_YY]

        # fig = plt.figure(figsize=(9,10))
        # fig.suptitle(f'MWA FEE MAP @ pointing [{p}] XX', fontsize=16, y=1.0)
        # plot_healpix(data_map=normed_beam_XX, sub=(1,1,1), cmap=jade, vmin=-50, vmax=0)
        # plt.savefig(f'{out_dir}/mwa_fee_beam_{p}_XX.png')
        # plt.close()

        fig = plt.figure(figsize=(9, 10))
        fig.suptitle(f"MWA FEE MAP @ pointing [{p}] YY", fontsize=16, y=1.0)
        plot_healpix(
            data_map=normed_beam_YY, sub=(1, 1, 1), cmap=jade, vmin=-50, vmax=0
        )
        plt.savefig(f"{out_dir}/mwa_fee_beam_{p}_YY_9_flagged.png")
        plt.close()

    np.savez_compressed(f"{out_dir}/mwa_fee_beam_9_flagged.npz", **fee_beam_flagged)



import sys
import math
import json
import pytz
import json
import numpy as np
from pathlib import Path
from astropy.time import Time
from datetime import datetime, timedelta


def time_tree(start_date, stop_date, time_zone):
    "Start/end of 30 minuts obs in local, unix, gps"

    # The time input is in local time. As in Austraila/Perth
    local = pytz.timezone(time_zone)

    t_start = datetime.strptime(start_date, "%Y-%m-%d")
    t_stop = datetime.strptime(stop_date, "%Y-%m-%d")

    # Number of days that the date range spans
    n_days = (t_stop - t_start).days

    # YYYY-MM-DD-HH:MM format
    obs_time = []

    # Start of half hour obs in unix time
    obs_unix = []

    # End of half hour obs in unix time
    obs_unix_end = []

    # The +1 makes the date ranges inclusive
    for i in range(n_days + 1):
        day = t_start + timedelta(days=i)
        date = day.strftime("%Y-%m-%d")

        # loop over 48x30 minute obs in a day
        for j in range(48):
            t_delta = datetime.strptime(date, "%Y-%m-%d") + timedelta(minutes=30 * j)
            # convert t_delta to a readable string YYYY-MM-DD-HH:MM
            d_time = t_delta.strftime("%Y-%m-%d-%H:%M")

            # Convert from naive local time to utc aware time
            utc_delta = local.localize(t_delta, is_dst=None).astimezone(pytz.utc)

            # convert to a unix timestamp, used within rf explorer data files
            utc_unix = utc_delta.timestamp()
            # time at end of half hour window
            utc_unix_end = utc_unix + (30 * 60)

            obs_time.append(d_time)
            obs_unix.append(utc_unix)
            obs_unix_end.append(utc_unix_end)

    # Start and end of 30 min obs in gps time
    # Round to nearest int
    obs_gps = np.rint(Time(obs_unix, format="unix").gps)
    obs_gps_end = np.rint(Time(obs_unix_end, format="unix").gps)

    return (obs_time, obs_unix, obs_unix_end, obs_gps, obs_gps_end)


def read_json(out_dir, f_name):
    """Read ultimate_pointing_times.json"""

    with open(f"{out_dir}/{f_name}") as table:
        data = json.load(table)
        pointings = data["grid_pt"]
        start_gps = data["start_gps"]
        stop_gps = data["stop_gps"]
        obs_length = data["obs_length"]

    return (pointings, start_gps, stop_gps, obs_length)


def time_filter(s_rise, s_set, times):
    """How much of the 30 minute
    window was at a particular 
    pointing"""

    # I. sat rises before times, sets within times window
    if s_rise < times[0] and s_set > times[0] and s_set <= times[-1]:
        occu = (s_set - times[0]) / 1800

    # II. sat rises and sets within times
    elif s_rise >= times[0] and s_set <= times[-1]:
        occu = (s_set - s_rise) / 1800

    # III. sat rises within times, and sets after
    elif s_rise >= times[0] and s_rise < times[-1] and s_set > times[-1]:
        occu = (times[-1] - s_rise) / 1800

    # IV. sat rises before times and sets after
    elif s_rise < times[0] and s_set > times[-1]:
        occu = (times[-1] - times[0]) / 1800

    # V. sat completely out of times. Could be on either side
    else:
        occu = None

    return occu


def obs_pointing(
    obs_time,
    obs_gps,
    obs_gps_end,
    pointings,
    start_gps,
    stop_gps,
    start_date,
    stop_date,
    out_dir,
):

    """Find pointing of 30 min obs
    Classification of more than 70%
    of an obs is at one pointing"""

    point_0 = []
    point_2 = []
    point_4 = []
    point_41 = []

    # loop over each 30 min observation
    for i in range(len(obs_time)):

        # start, end times in gps format
        obs_window = [obs_gps[i], obs_gps_end[i]]

        # loop over mwa pointing observations from
        # ultimate_pointing_times.json
        for j in range(len(start_gps)):
            occu = time_filter(start_gps[j], stop_gps[j], obs_window)

            if occu != None and occu >= 0.7:
                if pointings[j] == 0:
                    point_0.append(obs_time[i])
                elif pointings[j] == 2:
                    point_2.append(obs_time[i])
                elif pointings[j] == 4:
                    point_4.append(obs_time[i])
                elif pointings[j] == 41:
                    point_41.append(obs_time[i])
                else:
                    pass

    # Create dictionary to be saved to json
    obs_pointings = {}

    obs_pointings["start_date"] = start_date
    obs_pointings["stop_date"] = stop_date
    obs_pointings["point_0"] = point_0
    obs_pointings["point_2"] = point_2
    obs_pointings["point_4"] = point_4
    obs_pointings["point_41"] = point_41

    with open(f"{out_dir}/obs_pointings.json", "w") as outfile:
        json.dump(obs_pointings, outfile, indent=4)


if __name__ == "__main__":

    import argparse

    parser = argparse.ArgumentParser(
        description="""
            Classify the pointings of 30 min observations
            """
    )

    parser.add_argument(
        "--start_date",
        metavar="\b",
        help="Date from which to determine sat ephemeris. Ex: 2019-10-10",
    )
    parser.add_argument(
        "--stop_date",
        metavar="\b",
        help="Date until which to determine sat ephemeris. Ex: 2019-10-11",
    )
    parser.add_argument(
        "--time_zone",
        metavar="\b",
        default="Australia/Perth",
        help="Time zone where data was recorded. Default=Australia/Perth",
    )
    parser.add_argument(
        "--out_dir",
        metavar="\b",
        default="./../../outputs/beam_pointings/",
        help="Directory where json metadata files live. Default=./../../outputs/beam_pointings/",
    )
    parser.add_argument(
        "--f_name",
        metavar="\b",
        default="ultimate_pointing_times.json",
        help="File name of json to be plotted. Default=ultimate_pointing_times.json",
    )

    args = parser.parse_args()
    start_date = args.start_date
    stop_date = args.stop_date
    time_zone = args.time_zone
    out_dir = Path(args.out_dir)
    f_name = Path(args.f_name)

    obs_time, obs_unix, obs_unix_end, obs_gps, obs_gps_end = time_tree(
        start_date, stop_date, time_zone
    )

    pointings, start_gps, stop_gps, obs_length = read_json(out_dir, f_name)

    obs_pointing(
        obs_time,
        obs_gps,
        obs_gps_end,
        pointings,
        start_gps,
        stop_gps,
        start_date,
        stop_date,
        out_dir,
    )



import sys
import json
import argparse
import matplotlib
import numpy as np

matplotlib.use("Agg")
from pathlib import Path
import scipy.optimize as opt
import matplotlib.pyplot as plt
import matplotlib.gridspec as gs
from scipy.stats import median_absolute_deviation as mad
from mpl_toolkits.axes_grid1 import make_axes_locatable
import numpy.polynomial.polynomial as poly
from scipy.stats import binned_statistic
import scipy.optimize as opt

sys.path.append("../decode_rf_data")
from colormap import spectral

cmap = spectral()

parser = argparse.ArgumentParser(
    description="""
    Liner Fit for RF Explorer Gain variations
    """
)

parser.add_argument(
    "--rfe_dir",
    metavar="\b",
    default="../../outputs/tile_maps/rfe_gain/",
    help="Dir where RFE residual json files live. Default:../../outputs/tile_maps/rfe_gain/",
)

parser.add_argument(
    "--start_gain",
    metavar="\b",
    default=-50,
    type=int,
    help="Power at which RFE gain variations begin. Default: -50dBm",
)
parser.add_argument(
    "--stop_gain",
    metavar="\b",
    default=-30,
    type=int,
    help="Power at which RFE gain variations saturate. Default: -30dBm",
)

args = parser.parse_args()

start_gain = args.start_gain
stop_gain = args.stop_gain
rfe_dir = Path(args.rfe_dir)

# find all rfe_gain json files
gain_files = [item for item in rfe_dir.glob("*.json")]
names = [i.name.split(".")[0] for i in gain_files]

# Combine data from all RF Explorers
pass_data = []
pass_resi = []

for n, f in enumerate(gain_files):
    with open(f, "r") as data:
        rfe = json.load(data)
        pass_data.extend(rfe["pass_data"])
        pass_resi.extend(rfe["pass_resi"])

        # pass_data = rfe['pass_data']
        # pass_resi = rfe['pass_resi']
        #
        # plt.style.use('seaborn')
        # fig = plt.figure()
        # plt.scatter(pass_data, pass_resi, marker='.', alpha=0.7, color='seagreen')
        # plt.xlabel('Observed power [dBm]')
        # plt.ylabel('Residuals power [dB]')
        # plt.xlim([-80,-20])
        # plt.ylim([-20,20])
        # plt.tight_layout()
        # plt.savefig(f'../../outputs/paper_plots/{names[n]}.png', bbox_inches='tight')
        # plt.close()


fig = plt.figure()

plt.hexbin(pass_data, pass_resi, gridsize=121, cmap=cmap, alpha=0.99, zorder=0)

pass_data = np.array(pass_data)
pass_resi = np.array(pass_resi)

# Clean up huge noisy outliers
filtr = np.where(np.logical_and(pass_data <= -25, pass_data >= -65))
pass_data = pass_data[filtr]
pass_resi = pass_resi[filtr]

# Median of binned data
bin_med, bin_edges, binnumber = binned_statistic(
    pass_data, pass_resi, statistic="median", bins=16
)
bin_width = bin_edges[1] - bin_edges[0]
bin_centers = bin_edges[1:] - bin_width / 2

# Now look at data between -50, -30, where gains vary
filtr = np.where(np.logical_and(pass_data >= start_gain, pass_data <= stop_gain))
pass_data = pass_data[filtr]
pass_resi = pass_resi[filtr]

# Linear fit to RFE data between -50, -30 dBm
poly = np.polyfit(pass_data, pass_resi, 2)
# Mathematical function of fit, which can be evaluated anywhere
f = np.poly1d(poly)
# save polyfit to file
np.save(f"{rfe_dir}/rfe_gain_fit.npy", poly)

# x_f = [f.roots[0], -45, -40, -35, -30, -25, -20]
x_f = np.linspace(max(f.roots), -25, num=10)
y_f = f(x_f)

plt.plot(
    x_f,
    y_f,
    color="w",
    lw=2.1,
    marker="s",
    markeredgecolor="k",
    markeredgewidth=1.6,
    markersize=4.9,
    alpha=1,
    label="Gain fit",
    zorder=1,
)
plt.scatter(
    bin_centers,
    bin_med,
    marker="X",
    s=36,
    facecolors="#ee4540",
    lw=0.9,
    edgecolors="w",
    alpha=1,
    label="Median residuals",
    zorder=2,
)

leg = plt.legend(loc="lower right", frameon=True, markerscale=0.9, handlelength=1.4)
leg.get_frame().set_facecolor("#cccccc")
for l in leg.legendHandles:
    l.set_alpha(0.77)


plt.xlabel("Observed power [dBm]")
plt.ylabel("Residuals power [dB]")
plt.xlim([-65, -25])
plt.ylim([-10, 15])
plt.tick_params(axis="both", length=0)
plt.grid(color="#cccccc", alpha=0.36, lw=1.2)
plt.box(None)
plt.tight_layout()
plt.savefig(f"{rfe_dir}/rfe_gain_fit.png", bbox_inches="tight")



# Package imports
from setuptools import setup, find_packages
from pathlib import Path
import re

# Get long description from README
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

# Get the requirements list
with open("requirements.txt", "r") as f:
    requirements = f.read().splitlines()

# Recursive data files in embers.kindle
def data_files(data_dir, data_types):
    """Returns a list of data files"""

    data_dir = Path(data_dir)
    data_files = []
    for d in data_types:
        files = [p.relative_to(Path(f"embers")) for p in data_dir.rglob(d)]
        data_files.extend(files)
    return [p.as_posix() for p in data_files]


# Obtain name, version, author from __init__.py file
from embers import __version__, __author__

version = __version__
author = __author__


setup(
    name="embers",
    version=version,
    license="MIT",
    author=author,
    author_email="achokshi@student.unimelb.edu.au",
    description="Experimental Measurement of BEam Response with Satellites",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/amanchokshi/EMBERS",
    packages=find_packages(),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Operating System :: MacOS",
        "Operating System :: Unix",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Scientific/Engineering :: Astronomy",
        "Topic :: Scientific/Engineering :: Physics",
    ],
    entry_points={
        "console_scripts": [
            "colormaps=embers.kindle.colormaps:main",
            "waterfall=embers.kindle.waterfall:main",
            "waterfall_batch=embers.kindle.waterfall_batch:main",
            "align_single=embers.kindle.align_single:main",
            "align_batch=embers.kindle.align_batch:main",
        ],
    },
    keywords=("embers radio astronomy satellites beam measurement"),
    package_data={"": data_files("embers/kindle/data", ["*.txt", "*.ffe"])},
    install_requires=requirements,
    python_requires=">=3.6, <4",
    include_package_data=True,
    zip_safe=False,
)



import os
import sys
import json
import argparse
from itertools import repeat
import sat_ephemeris as se
import sat_ids

import numpy as np
import skyfield as sf
from pathlib import Path
import concurrent.futures
from astropy.time import Time
from skyfield.api import Topos, load


# Save ephem date for one satellite
def sat_json(tle_dir, cadence, out_dir, sat_id):
    try:
        tle_path = f"{tle_dir}/{sat_id}.txt"

        sat_ephem = {}
        sat_ephem["sat_id"] = [sat_id]
        sat_ephem["time_array"] = []
        sat_ephem["sat_alt"] = []
        sat_ephem["sat_az"] = []

        sats, epochs = se.load_tle(tle_path)
        epoch_range = se.epoch_ranges(epochs)

        for i in range(len(epoch_range) - 1):
            try:
                t_arr, index_epoch = se.epoch_time_array(epoch_range, i, cadence)
                passes, alt, az = se.sat_pass(sats, t_arr, index_epoch)

                for pass_index in passes:
                    time_array, sat_alt, sat_az = se.ephem_data(
                        t_arr, pass_index, alt, az
                    )

                    time_array = list(time_array)

                    # We don't care about sat passes shorter than a minute (3*20sec)
                    if len(time_array) >= 3:
                        sat_ephem["time_array"].append(time_array)
                        sat_ephem["sat_alt"].append(sat_alt)
                        sat_ephem["sat_az"].append(sat_az)
            except Exception:
                pass
        with open(f"{out_dir}/{sat_id}.json", "w") as outfile:
            json.dump(sat_ephem, outfile, indent=4)

        return f"Saved {sat_id}.json"
    except Exception:
        return f"ERROR! Couldn't save {sat_id}.json. TLE was empty!"


if __name__ == "__main__":

    # Skyfield Timescale
    ts = load.timescale(builtin=True)

    # Position of MWA site in Lat/Lon/Elevation
    MWA = Topos(latitude=-26.703319, longitude=116.670815, elevation_m=337.83)

    parser = argparse.ArgumentParser(
        description="""
            Code which converts the TLE files downloaded with download_TLE.py
            into satellite ephemeris data: rise time, set time, alt/az arrays
            at a given time cadence. This is saved to a json file which will 
            be used to plot the satellite passes.
            """
    )

    parser.add_argument(
        "--tle_dir",
        metavar="\b",
        default="./../../outputs/sat_ephemeris/TLE",
        help="Directory where TLE files are saved. Default=./../../outputs/sat_ephemeris/TLE",
    )
    parser.add_argument(
        "--cadence",
        metavar="\b",
        type=int,
        default=4,
        help="Rate at which sat alt/az is computed. Default=4s",
    )
    parser.add_argument(
        "--out_dir",
        metavar="\b",
        default="./../../outputs/sat_ephemeris/ephem_json/",
        help="Path to output directory. Default=./../../outputs/sat_ephemeris/ephem_json/",
    )

    args = parser.parse_args()

    tle_dir = args.tle_dir
    cadence = int(args.cadence)
    out_dir = args.out_dir

    # Load list of satellites (Nodar ids)
    sat_list = list(sat_ids.norad_ids.values())

    # Save logs
    Path(out_dir).mkdir(parents=True, exist_ok=True)
    sys.stdout = open(f"{out_dir}/sat_ephem_logs.txt", "a")

    # Parellization Magic Here!
    with concurrent.futures.ProcessPoolExecutor() as executor:
        results = executor.map(
            sat_json, repeat(tle_dir), repeat(cadence), repeat(out_dir), sat_list
        )

    for result in results:
        print(result)



import json
import numpy as np
from pathlib import Path
from astropy.io import fits
import matplotlib.pylab as pl
import matplotlib.pyplot as plt


def find_flag(metafits, tiles):

    # dictionary to save flag info.
    # 0 indicates no dead dipole
    # 1-16 dipole index
    flags = {}
    flags["obsid"] = []
    for t in tiles:
        for p in ["X", "Y"]:
            flags[f"{t}{p}"] = []

    meta_files = [item for item in metafits.glob("*.metafits")]
    for m in meta_files:

        hdu = fits.open(m)
        mode = hdu[0].header["MODE"]
        obsid = hdu[0].header["GPSTIME"]
        delays = hdu[1].data["Delays"]

        tile_names = hdu[1].data["TileName"]
        delays = hdu[1].data["Delays"]
        pols = hdu[1].data["Pol"]

        if mode == "HW_LFILES":

            flags["obsid"].append(int(obsid))

            for t in tiles:
                idx = np.where(tile_names == t)
                for i in range(2):
                    idx_p = idx[0][i]

                    t_name = tile_names[idx_p]
                    t_pol = pols[idx_p]
                    t_del = delays[idx_p]
                    t_flag = np.where(t_del == 32)[0] + 1

                    if t_flag.size != 0:
                        flags[f"{t_name}{t_pol}"].append(int(t_flag[0]))
                    else:
                        flags[f"{t_name}{t_pol}"].append(0)

        hdu.close()

    return flags


if __name__ == "__main__":

    import argparse

    parser = argparse.ArgumentParser(
        description="""
            Look for dead dipoles
            """
    )

    parser.add_argument(
        "--metafits",
        metavar="\b",
        default="./../../outputs/beam_pointings/metafits/",
        help="Directory where metafits files live. Default=./../../outputs/beam_pointings/metafits/",
    )

    parser.add_argument(
        "--out_dir",
        metavar="\b",
        default="./../../outputs/beam_pointings/",
        help="Directory where metafits files live. Default=./../../outputs/beam_pointings/",
    )

    args = parser.parse_args()
    metafits = Path(args.metafits)
    out_dir = Path(args.out_dir)

    # list of all tiles used in this experiment
    tiles = [
        "HexS6",
        "HexS7",
        "HexS8",
        "HexS9",
        "HexS10",
        "HexS12",
        "HexS29",
        "HexS30",
        "HexS31",
        "HexS32",
        "HexS33",
        "HexS34",
        "HexS35",
        "HexS36",
    ]

    flags = find_flag(metafits, tiles)

    # for m in meta_files:
    #    find_flag(m)

    # Save flagging info to json file
    with open(f"{out_dir}/flags.json", "w") as outfile:
        json.dump(flags, outfile, indent=4)

    keys = list(flags.keys())

    n = len(keys) - 1
    colors = pl.cm.Spectral(np.linspace(0, 1, n))

    plt.style.use("seaborn")

    fig, axs = plt.subplots(4, 7, figsize=(18, 9), sharex=True, sharey=True,)
    axs = axs.ravel()

    for i in range(n):
        axs[i].scatter(
            flags["obsid"],
            flags[keys[i + 1]],
            color=colors[i],
            linewidths=0.1,
            s=28,
            alpha=0.88,
            label=keys[i + 1],
        )
        axs[i].set_ylim(-1, 17)
        axs[i].set_yticks([0, 4, 8, 12, 16])

        # these are matplotlib.patch.Patch properties
        props = dict(boxstyle="round", facecolor="wheat", alpha=0.2)

        # place a text box in upper left in axes coords
        axs[i].text(
            0.67,
            0.93,
            f"{keys[i+1]}",
            transform=axs[i].transAxes,
            fontsize=10,
            verticalalignment="top",
            bbox=props,
        )

    plt.tight_layout()
    plt.savefig(f"{out_dir}/dead_dipoles.png")



"""
============
Align Batch
============

Interpolate and Savitzky–Golay smooth a pair of raw rf data files
before temorally aligning the. Save aligned data as npz files to 
``./embers_out/condition_data/align_data``

"""

import logging
import argparse
import pkg_resources
from pathlib import Path
import concurrent.futures
from itertools import repeat
import matplotlib.pyplot as plt
from embers.condition_data.align_data import save_aligned
from embers.condition_data.rf_data import tile_names, tile_pairs, time_tree

_parser = argparse.ArgumentParser(
    description="""
    Batch align pairs of rf data files between a date interval
    """
)

_parser.add_argument(
    "--start_date", metavar="\b", default="", help="start date in YYYY-MM-DD format"
)

_parser.add_argument(
    "--stop_date", metavar="\b", default="", help="stop date in YYYY-MM-DD format"
)

_parser.add_argument(
    "--savgol_window_1",
    metavar="\b",
    type=int,
    default=11,
    help="First savgol window. Default=11",
)

_parser.add_argument(
    "--savgol_window_2",
    metavar="\b",
    type=int,
    default=15,
    help="Second savgol window. Default=15",
)

_parser.add_argument(
    "--polyorder", metavar="\b", type=int, default=2, help="Polynomial order. Default=2"
)

_parser.add_argument(
    "--interp_type",
    metavar="\b",
    default="cubic",
    help="Interpolation type. Default=cubic",
)

_parser.add_argument(
    "--interp_freq",
    metavar="\b",
    type=int,
    default=1,
    help="Interpolation frequency. Default=1",
)

_parser.add_argument(
    "--data_dir", metavar="\b", default="", help="root of dir where rf data is saved"
)

_parser.add_argument(
    "--out_dir",
    metavar="\b",
    default="./embers_out/condition_data/align_data",
    help="Dir where savgol-interp sample plot is saved. Default=./embers_out/condition_data/align_data",
)

_args = _parser.parse_args()
_start_date = _args.start_date
_stop_date = _args.stop_date
_savgol_window_1 = _args.savgol_window_1
_savgol_window_2 = _args.savgol_window_2
_polyorder = _args.polyorder
_interp_type = _args.interp_type
_interp_freq = _args.interp_freq
_data_dir = _args.data_dir
_out_dir = _args.out_dir


# if no input file provided, use sample package data
if _data_dir == "":
    print("----------------------------------------------------------")
    print("No data dir provided, using packaged sample data")
    _data_dir = pkg_resources.resource_filename("embers.kindle", "data/rf_data/")

if _start_date == "":
    print("No start_date provided, using 2019-10-01 for sample data")
    _start_date = "2019-10-01"

if _stop_date == "":
    print("No stop_date provided, using 2019-10-10 for sample data")
    print("")
    print(">>> align_batch --help, for more options")
    print("----------------------------------------------------------")
    _stop_date = "2019-10-10"


# Logging config
_log_dir = Path(f"{_out_dir}")
_log_dir.mkdir(parents=True, exist_ok=True)
logging.basicConfig(
    filename=f"{_out_dir}/align_batch.log",
    level=logging.INFO,
    format="%(levelname)s: %(funcName)s: %(message)s",
)


def align_batch(
    start_date=None,
    stop_date=None,
    savgol_window_1=None,
    savgol_window_2=None,
    polyorder=None,
    interp_type=None,
    interp_freq=None,
    data_dir=None,
    out_dir=None,
):

    dates, time_stamps = time_tree(start_date, stop_date)

    for pair in tile_pairs(tile_names()):

        for day in range(len(dates)):

            with concurrent.futures.ProcessPoolExecutor() as executor:
                results = executor.map(
                    save_aligned,
                    repeat(pair),
                    time_stamps[day],
                    repeat(savgol_window_1),
                    repeat(savgol_window_2),
                    repeat(polyorder),
                    repeat(interp_type),
                    repeat(interp_freq),
                    repeat(data_dir),
                    repeat(out_dir),
                )

            for result in results:
                logging.info(result)


def main():
    """Execute align_batch from terminal."""

    print(f"Aligned files saved to: {_out_dir}")
    align_batch(
        start_date=_start_date,
        stop_date=_stop_date,
        savgol_window_1=_savgol_window_1,
        savgol_window_2=_savgol_window_2,
        polyorder=_polyorder,
        interp_type=_interp_type,
        interp_freq=_interp_freq,
        data_dir=_data_dir,
        out_dir=_out_dir,
    )



import numpy as np
import healpy as hp
import scipy.optimize as opt
import numpy.polynomial.polynomial as poly


def hp_slices_horizon(nside=None):
    """Healpix pix indices of NS, EW slices and above horizon"""
    # theta phi values of each pixel
    hp_indices = np.arange(hp.nside2npix(nside))
    θ, ɸ = hp.pix2ang(nside, hp_indices)

    # healpix indices above the horizon
    above_horizon_indices = np.where(θ <= np.radians(80))[0]

    # pixel coords above the horizon
    ɸ_above_horizon = ɸ[above_horizon_indices]

    NS_indices = []
    EW_indices = []

    # pixel indices along N, E, S, W slices
    # order the indices such that they proceed from N -> S or E -> W
    n_slice = sorted(
        np.where((np.round(np.degrees(ɸ_above_horizon))) == 45)[0], reverse=True
    )
    e_slice = sorted(
        np.where((np.round(np.degrees(ɸ_above_horizon))) == 135)[0], reverse=True
    )
    s_slice = sorted(np.where((np.round(np.degrees(ɸ_above_horizon))) == 225)[0])
    w_slice = sorted(np.where((np.round(np.degrees(ɸ_above_horizon))) == 315)[0])

    NS_indices.extend(n_slice)
    NS_indices.extend(s_slice)
    EW_indices.extend(e_slice)
    EW_indices.extend(w_slice)

    return [NS_indices, EW_indices, above_horizon_indices]


def slice_map(hp_map):
    """slices healpix map along NS, EW"""

    NS_indices, EW_indices, _ = hp_slices_horizon(nside)

    θ_NS, ɸ_NS = np.degrees(hp.pix2ang(nside, NS_indices))
    θ_EW, ɸ_EW = np.degrees(hp.pix2ang(nside, EW_indices))

    zenith_angle_NS = []
    for i, j in zip(θ_NS, ɸ_NS):
        if j <= 180:
            zenith_angle_NS.append(-1 * i)
        else:
            zenith_angle_NS.append(i)

    zenith_angle_EW = []
    for i, j in zip(θ_EW, ɸ_EW):
        if j <= 180:
            zenith_angle_EW.append(-1 * i)
        else:
            zenith_angle_EW.append(i)

    NS_data = [hp_map[NS_indices], zenith_angle_NS]
    EW_data = [hp_map[EW_indices], zenith_angle_EW]

    return [NS_data, EW_data]


def nan_mad(ref_map):
    """Compute mad while ignoring nans"""
    ref_map_mad = []
    for j in ref_map:
        if j != []:
            j = np.asarray(j)
            j = j[~np.isnan(j)]
            ref_map_mad.append(mad(j))
        else:
            ref_map_mad.append(np.nan)

    ref_map_mad = np.asarray(ref_map_mad)
    ref_map_mad[np.where(ref_map_mad == np.nan)] = np.nanmean(ref_map_mad)

    return ref_map_mad


def good_maps(ref_map):
    """Creates a ref map with only good satellites"""

    pointings = ["0", "2", "4"]

    # load data from map .npz file
    f = Path(f"{map_dir}/{ref_map}")
    tile_data = np.load(f, allow_pickle=True)
    tile_data = {key: tile_data[key].item() for key in tile_data}
    ref_map = tile_data["ref_map"]

    # Good sats from which to make plots
    good_sats = [
        25338,
        25984,
        25985,
        28654,
        40086,
        40087,
        40091,
        41179,
        41180,
        41182,
        41183,
        41184,
        41185,
        41187,
        41188,
        41189,
        44387,
    ]

    # Empty good map
    good_map = [[] for pixel in range(hp.nside2npix(nside))]

    for p in pointings:

        # append to good map from all good sat data
        for sat in good_sats:
            for pix in range(hp.nside2npix(nside)):
                good_map[pix].extend(ref_map[p][sat][pix])

    return good_map


def ref_map_slice(good_map):
    """slices ref healpix map along NS & EW"""

    ref_map_NS, ref_map_EW = slice_map(np.asarray(good_map))

    ref_med_map_NS = np.asarray(
        [(np.nanmedian(i) if i != [] else np.nan) for i in ref_map_NS[0]]
    )
    # Scale mean map such that the max value is 0
    ref_med_map_scaled_NS = np.asarray(
        [i - np.nanmax(ref_med_map_NS) for i in ref_med_map_NS]
    )
    # ref_mad_map_NS = np.asarray([mad(i) for i in ref_map_NS[0]])
    ref_mad_map_NS = np.asarray(nan_mad(ref_map_NS[0]))
    za_NS = ref_map_NS[1]

    ref_med_map_EW = np.asarray(
        [(np.nanmedian(i) if i != [] else np.nan) for i in ref_map_EW[0]]
    )
    # Scale mean map such that the max value is 0
    ref_med_map_scaled_EW = np.asarray(
        [i - np.nanmax(ref_med_map_EW) for i in ref_med_map_EW]
    )
    # ref_mad_map_EW = np.asarray([mad(i) for i in ref_map_EW[0]])
    ref_mad_map_EW = np.asarray(nan_mad(ref_map_EW[0]))
    za_EW = ref_map_EW[1]

    NS_data = [ref_med_map_scaled_NS, ref_mad_map_NS, za_NS]
    EW_data = [ref_med_map_scaled_EW, ref_mad_map_EW, za_EW]

    return [NS_data, EW_data]


# rotate func written by Jack Line
def rotate(nside, angle=None, healpix_array=None, savetag=None, flip=False):
    """Takes in a healpix array, rotates it by the desired angle, and saves it.
    Optionally flip the data, changes east-west into west-east because
    astronomy"""

    # theta phi values of each pixel
    hp_indices = np.arange(hp.nside2npix(nside))
    θ, ɸ = hp.pix2ang(nside, hp_indices)

    new_hp_inds = hp.ang2pix(nside, θ, ɸ + angle)

    ##Flip the data to match astro conventions
    if flip == True:
        new_angles = []
        for phi in ɸ:
            if phi <= np.pi:
                new_angles.append(np.pi - phi)
            else:
                new_angles.append(3 * np.pi - phi)
        new_hp_inds = hp.ang2pix(nside, ɸ, np.asarray(new_angles))

    ##Save the array in the new order
    if savetag:
        np.savez_compressed(savetag, beammap=healpix_array[new_hp_inds])

    return healpix_array[new_hp_inds]


# chisquared minimization to best fit map to data
def fit_gain(map_data=None, map_error=None, beam=None):
    """Fit the beam model to the measured data using
    chisquared minimization"""

    bad_values = np.isnan(map_data)
    map_data = map_data[~bad_values]
    map_error = map_error[~bad_values]

    map_error[np.where(map_error == 0)] = np.mean(map_error)

    def chisqfunc(gain):
        model = beam[~bad_values] + gain
        chisq = sum((map_data - model) ** 2)
        # chisq = sum(((map_data - model)/map_error)**2)
        return chisq

    x0 = np.array([0])

    result = opt.minimize(chisqfunc, x0)

    return result.x


def poly_fit(x, y, map_data, order):
    """Fit polynominal of order to data"""

    x = np.asarray(x)
    y = np.asarray(y)

    bad_values = np.isnan(map_data)
    x_good = x[~bad_values]
    y_good = y[~bad_values]
    coefs = poly.polyfit(x_good, y_good, order)
    fit = poly.polyval(x, coefs)
    return fit


def plt_slice(
    fig=None,
    sub=None,
    zen_angle=None,
    map_slice=None,
    map_error=None,
    model_slice=None,
    delta_pow=None,
    pow_fit=None,
    slice_label=None,
    model_label=None,
    ylabel=True,
):

    """Plot a slice of the beam, with measured
    data, errorbars, and fit the simulated beam
    to the data. Also plot the diff b/w data and
    the model"""

    ax = fig.add_subplot(sub)

    ax.errorbar(
        zen_angle,
        map_slice,
        yerr=map_error,
        fmt=".",
        color="#326765",
        ecolor="#7da87b",
        elinewidth=1.2,
        capsize=1.2,
        capthick=1.4,
        alpha=0.9,
        label=slice_label,
    )

    ax.plot(
        zen_angle,
        model_slice,
        color="#c70039",
        linewidth=1.2,
        alpha=0.9,
        label=model_label,
    )

    # ax.set_ylim(bottom=-30)
    # ax.set_xlabel('Zenith Angle (degrees)')
    # ax.legend(loc='lower center')
    leg = ax.legend(loc="lower center", frameon=True, handlelength=1)
    leg.get_frame().set_facecolor("white")
    for l in leg.legendHandles:
        l.set_alpha(1)
    ax.set_xlim([-82, 82])
    ax.set_ylim([-26, 12])

    divider = make_axes_locatable(ax)
    dax = divider.append_axes("bottom", size="30%", pad=0.1)

    # dax = fig.add_subplot(2,1,2)
    dax.scatter(zen_angle, delta_pow, marker=".", color="#27296d")
    dax.plot(zen_angle, pow_fit, linewidth=1.2, alpha=0.9, color="#ff8264")
    dax.set_xlim([-82, 82])
    dax.set_xticklabels([])
    dax.set_ylim([-5, 5])
    if ylabel is True:
        ax.set_ylabel("Power [dB]")
        dax.set_ylabel(r"$\Delta ref$ [dB]")
        # dax.set_ylabel('Data - Model (dB)')
    else:
        dax.set_yticklabels([])
        ax.set_yticklabels([])

    return ax


def plt_null_test(
    fig=None,
    sub=None,
    zen_angle=None,
    del_pow=None,
    del_err=None,
    del_beam=None,
    del_fit=None,
    null_label=None,
    beam_label=None,
    fit_label=None,
    ylabel=True,
):

    """Plot graphs for a null test"""

    ax = fig.add_subplot(sub)

    ax.errorbar(
        zen_angle,
        del_pow,
        yerr=del_err,
        fmt=".",
        color="#326765",
        ecolor="#7da87b",
        elinewidth=1.2,
        capsize=1.2,
        capthick=1.4,
        alpha=0.9,
        label=null_label,
    )

    ax.plot(
        zen_angle, del_beam, color="#c70039", linewidth=1.2, alpha=0.9, label=beam_label
    )

    ax.plot(
        zen_angle, del_fit, color="#ff8264", linewidth=1.2, alpha=0.9, label=fit_label
    )

    ax.set_xlim([-82, 82])
    ax.set_ylim([-10, 10])
    ax.set_xlabel("Zenith Angle [degrees]")
    # ax.legend(loc='upper left')
    leg = ax.legend(loc="upper left", frameon=True, handlelength=1)
    leg.get_frame().set_facecolor("white")
    for l in leg.legendHandles:
        l.set_alpha(1)

    if ylabel is True:
        ax.set_ylabel("Power [dB]")
    else:
        ax.set_yticklabels([])

    return ax


if __name__ == "__main__":

    import argparse
    import numpy as np
    from pathlib import Path
    import matplotlib

    matplotlib.use("Agg")
    import matplotlib.pyplot as plt
    import matplotlib.gridspec as gs
    from mpl_toolkits.axes_grid1 import make_axes_locatable
    from scipy.stats import median_absolute_deviation as mad

    import sys

    sys.path.append("../decode_rf_data")
    from plot_tile_maps import plot_healpix
    from colormap import spectral

    # Custom spectral colormap
    cmap = spectral()

    parser = argparse.ArgumentParser(
        description="""
        Plot healpix map of reference data
        """
    )

    parser.add_argument(
        "--out_dir",
        metavar="\b",
        default="../../outputs/tile_maps/null_test/",
        help="Output directory. Default=../../outputs/tile_maps/null_test/",
    )
    parser.add_argument(
        "--map_dir",
        metavar="\b",
        default="../../outputs/tile_maps/tile_maps_raw/",
        help="Output directory. Default=../../outputs/tile_maps/tile_maps_raw/",
    )
    parser.add_argument(
        "--ref_model",
        metavar="\b",
        default="../../outputs/reproject_ref/ref_dipole_models.npz",
        help="Healpix reference FEE model file. default=../../outputs/reproject_ref/ref_dipole_models.npz",
    )
    parser.add_argument(
        "--nside",
        metavar="\b",
        type=int,
        default=32,
        help="Healpix Nside. Default = 32",
    )

    args = parser.parse_args()

    out_dir = Path(args.out_dir)
    map_dir = Path(args.map_dir)
    ref_model = args.ref_model
    nside = args.nside

    out_dir.mkdir(parents=True, exist_ok=True)

    ref_tiles = [
        "S35XX_rf0XX_sat_maps.npz",
        "S35YY_rf0YY_sat_maps.npz",
        "S35XX_rf1XX_sat_maps.npz",
        "S35YY_rf1YY_sat_maps.npz",
    ]
    good_rf0XX = rotate(
        nside,
        angle=+(1 * np.pi) / 4.0,
        healpix_array=np.asarray(good_maps(ref_tiles[0])),
    )
    good_rf0YY = rotate(
        nside,
        angle=+(1 * np.pi) / 4.0,
        healpix_array=np.asarray(good_maps(ref_tiles[1])),
    )
    good_rf1XX = rotate(
        nside,
        angle=+(1 * np.pi) / 4.0,
        healpix_array=np.asarray(good_maps(ref_tiles[2])),
    )
    good_rf1YY = rotate(
        nside,
        angle=+(1 * np.pi) / 4.0,
        healpix_array=np.asarray(good_maps(ref_tiles[3])),
    )

    # NS, EW slices of all four reference tiles
    rf0XX_NS, rf0XX_EW = ref_map_slice(good_rf0XX)
    rf0YY_NS, rf0YY_EW = ref_map_slice(good_rf0YY)
    rf1XX_NS, rf1XX_EW = ref_map_slice(good_rf1XX)
    rf1YY_NS, rf1YY_EW = ref_map_slice(good_rf1YY)

    # Null test diff in power b/w rf0 & rf1
    ref01_XX_NS = rf0XX_NS[0] - rf1XX_NS[0]
    ref01_XX_EW = rf0XX_EW[0] - rf1XX_EW[0]
    ref01_YY_NS = rf0YY_NS[0] - rf1YY_NS[0]
    ref01_YY_EW = rf0YY_EW[0] - rf1YY_EW[0]

    # Error propogation in null test
    error_ref01_XX_NS = np.sqrt((rf0XX_NS[1]) ** 2 + (rf1XX_NS[1]) ** 2)
    error_ref01_XX_EW = np.sqrt((rf0XX_EW[1]) ** 2 + (rf1XX_EW[1]) ** 2)
    error_ref01_YY_NS = np.sqrt((rf0YY_NS[1]) ** 2 + (rf1YY_NS[1]) ** 2)
    error_ref01_YY_EW = np.sqrt((rf0YY_EW[1]) ** 2 + (rf1YY_EW[1]) ** 2)

    # Load reference FEE model
    ref_fee_model = np.load(ref_model, allow_pickle=True)
    beam_XX = ref_fee_model["XX"]
    beam_YY = ref_fee_model["YY"]

    # Rotate beam models by pi/4 to match rotation of data
    rotated_XX = rotate(nside, angle=-(1 * np.pi) / 4.0, healpix_array=beam_XX)
    rotated_YY = rotate(nside, angle=-(1 * np.pi) / 4.0, healpix_array=beam_YY)

    # These plots show that the pi/4 rotation was correct
    # plot_healpix(data_map=rotated_XX,sub=(1,1,1), cmap=cmap, vmin=-40, vmax=-20)
    # plot_healpix(data_map=rotated_YY,sub=(1,1,1), cmap=cmap, vmin=-40, vmax=-20)
    # plt.show()

    # slice the XX rotated map along NS, EW
    XX_NS, XX_EW = slice_map(rotated_XX)
    XX_NS_slice, za_NS = XX_NS
    XX_EW_slice, za_EW = XX_EW

    # slice the YY rotated map along NS, EW
    YY_NS, YY_EW = slice_map(rotated_YY)
    YY_NS_slice, za_NS = YY_NS
    YY_EW_slice, za_EW = YY_EW

    # Gain offsets for the 8 combinations of data and beam slices
    gain_ref0_XX_NS = fit_gain(
        map_data=rf0XX_NS[0], map_error=rf0XX_NS[1], beam=XX_NS_slice
    )
    gain_ref0_XX_EW = fit_gain(
        map_data=rf0XX_EW[0], map_error=rf0XX_EW[1], beam=XX_EW_slice
    )
    gain_ref1_XX_NS = fit_gain(
        map_data=rf1XX_NS[0], map_error=rf1XX_NS[1], beam=XX_NS_slice
    )
    gain_ref1_XX_EW = fit_gain(
        map_data=rf1XX_EW[0], map_error=rf1XX_EW[1], beam=XX_EW_slice
    )

    gain_ref0_YY_NS = fit_gain(
        map_data=rf0YY_NS[0], map_error=rf0YY_NS[1], beam=YY_NS_slice
    )
    gain_ref0_YY_EW = fit_gain(
        map_data=rf0YY_EW[0], map_error=rf0YY_EW[1], beam=YY_EW_slice
    )
    gain_ref1_YY_NS = fit_gain(
        map_data=rf1YY_NS[0], map_error=rf1YY_NS[1], beam=YY_NS_slice
    )
    gain_ref1_YY_EW = fit_gain(
        map_data=rf1YY_EW[0], map_error=rf1YY_EW[1], beam=YY_EW_slice
    )

    # Scale the data so that it best fits the beam slice
    rf0XX_NS = rf0XX_NS - gain_ref0_XX_NS
    rf0XX_EW = rf0XX_EW - gain_ref0_XX_EW
    rf0YY_NS = rf0YY_NS - gain_ref0_YY_NS
    rf0YY_EW = rf0YY_EW - gain_ref0_YY_EW
    rf1XX_NS = rf1XX_NS - gain_ref1_XX_NS
    rf1XX_EW = rf1XX_EW - gain_ref1_XX_EW
    rf1YY_NS = rf1YY_NS - gain_ref1_YY_NS
    rf1YY_EW = rf1YY_EW - gain_ref1_YY_EW

    # Difference b/w beam model slices.
    # Always 0 because we have only one model for both rf0, rf1
    beam_ref01_XX_NS = XX_NS_slice - XX_NS_slice
    beam_ref01_XX_EW = XX_EW_slice - XX_EW_slice
    beam_ref01_YY_NS = YY_NS_slice - YY_NS_slice
    beam_ref01_YY_EW = YY_EW_slice - YY_EW_slice

    # delta powers
    del_pow_ref0_XX_NS = rf0XX_NS[0] - XX_NS_slice
    del_pow_ref0_XX_EW = rf0XX_EW[0] - XX_EW_slice
    del_pow_ref1_XX_NS = rf1XX_NS[0] - XX_NS_slice
    del_pow_ref1_XX_EW = rf1XX_EW[0] - XX_EW_slice

    del_pow_ref0_YY_NS = rf0YY_NS[0] - YY_NS_slice
    del_pow_ref0_YY_EW = rf0YY_EW[0] - YY_EW_slice
    del_pow_ref1_YY_NS = rf1YY_NS[0] - YY_NS_slice
    del_pow_ref1_YY_EW = rf1YY_EW[0] - YY_EW_slice

    # 3rd order poly fits for residuals
    fit_ref0_XX_NS = poly_fit(za_NS, del_pow_ref0_XX_NS, rf0XX_NS[0], 3)
    fit_ref0_XX_EW = poly_fit(za_EW, del_pow_ref0_XX_EW, rf0XX_EW[0], 3)
    fit_ref1_XX_NS = poly_fit(za_NS, del_pow_ref1_XX_NS, rf1XX_NS[0], 3)
    fit_ref1_XX_EW = poly_fit(za_EW, del_pow_ref1_XX_EW, rf1XX_EW[0], 3)

    fit_ref0_YY_NS = poly_fit(za_NS, del_pow_ref0_YY_NS, rf0YY_NS[0], 3)
    fit_ref0_YY_EW = poly_fit(za_EW, del_pow_ref0_YY_EW, rf0YY_EW[0], 3)
    fit_ref1_YY_NS = poly_fit(za_NS, del_pow_ref1_YY_NS, rf1YY_NS[0], 3)
    fit_ref1_YY_EW = poly_fit(za_EW, del_pow_ref1_YY_EW, rf1YY_EW[0], 3)

    # Difference of power b/w ref0 & ref1 fits
    fit_ref01_XX_NS = fit_ref0_XX_NS - fit_ref1_XX_NS
    fit_ref01_XX_EW = fit_ref0_XX_EW - fit_ref1_XX_EW
    fit_ref01_YY_NS = fit_ref0_YY_NS - fit_ref1_YY_NS
    fit_ref01_YY_EW = fit_ref0_YY_EW - fit_ref1_YY_EW

    plt.style.use("seaborn")
    fig1 = plt.figure(figsize=(8, 10))

    ax1 = plt_slice(
        fig=fig1,
        sub=321,
        zen_angle=za_NS,
        map_slice=rf0XX_NS[0],
        map_error=rf0XX_NS[1],
        model_slice=XX_NS_slice,
        delta_pow=del_pow_ref0_XX_NS,
        pow_fit=fit_ref0_XX_NS,
        slice_label="ref0XX NS",
        model_label="FEE XX NS",
    )

    ax2 = plt_slice(
        fig=fig1,
        sub=322,
        zen_angle=za_EW,
        map_slice=rf0XX_EW[0],
        map_error=rf0XX_EW[1],
        model_slice=XX_EW_slice,
        delta_pow=del_pow_ref0_XX_EW,
        pow_fit=fit_ref0_XX_EW,
        slice_label="ref0XX EW",
        model_label="FEE XX EW",
        ylabel=False,
    )

    ax3 = plt_slice(
        fig=fig1,
        sub=323,
        zen_angle=za_NS,
        map_slice=rf1XX_NS[0],
        map_error=rf1XX_NS[1],
        model_slice=XX_NS_slice,
        delta_pow=del_pow_ref1_XX_NS,
        pow_fit=fit_ref1_XX_NS,
        slice_label="ref1XX NS",
        model_label="FEE XX NS",
    )

    ax4 = plt_slice(
        fig=fig1,
        sub=324,
        zen_angle=za_EW,
        map_slice=rf1XX_EW[0],
        map_error=rf1XX_EW[1],
        model_slice=XX_EW_slice,
        delta_pow=del_pow_ref1_XX_EW,
        pow_fit=fit_ref1_XX_EW,
        slice_label="ref1XX EW",
        model_label="FEE XX EW",
        ylabel=False,
    )

    ax5 = plt_null_test(
        fig=fig1,
        sub=325,
        zen_angle=za_NS,
        del_pow=ref01_XX_NS,
        del_err=error_ref01_XX_NS,
        del_beam=beam_ref01_XX_NS,
        del_fit=fit_ref01_XX_NS,
        null_label="NS rf0-rf1",
        beam_label="FEE Null",
        fit_label="Fit rf0-rf1",
    )

    ax6 = plt_null_test(
        fig=fig1,
        sub=326,
        zen_angle=za_EW,
        del_pow=ref01_XX_EW,
        del_err=error_ref01_XX_EW,
        del_beam=beam_ref01_XX_EW,
        del_fit=fit_ref01_XX_EW,
        null_label="EW rf0-rf1",
        beam_label="FEE Null",
        fit_label="Fit rf0-rf1",
        ylabel=False,
    )

    plt.tight_layout()
    fig1.savefig(f"{out_dir}/null_test_XX_slices.png")

    fig2 = plt.figure(figsize=(8, 10))

    ax7 = plt_slice(
        fig=fig2,
        sub=321,
        zen_angle=za_NS,
        map_slice=rf0YY_NS[0],
        map_error=rf0YY_NS[1],
        model_slice=YY_NS_slice,
        delta_pow=del_pow_ref0_YY_NS,
        pow_fit=fit_ref0_YY_NS,
        slice_label="ref0YY NS",
        model_label="FEE YY NS",
    )

    ax8 = plt_slice(
        fig=fig2,
        sub=322,
        zen_angle=za_EW,
        map_slice=rf0YY_EW[0],
        map_error=rf0YY_EW[1],
        model_slice=YY_EW_slice,
        delta_pow=del_pow_ref0_YY_EW,
        pow_fit=fit_ref0_YY_EW,
        slice_label="ref0YY EW",
        model_label="FEE YY EW",
        ylabel=False,
    )

    ax9 = plt_slice(
        fig=fig2,
        sub=323,
        zen_angle=za_NS,
        map_slice=rf1YY_NS[0],
        map_error=rf1YY_NS[1],
        model_slice=YY_NS_slice,
        delta_pow=del_pow_ref1_YY_NS,
        pow_fit=fit_ref1_YY_NS,
        slice_label="ref1YY NS",
        model_label="FEE YY NS",
    )

    ax10 = plt_slice(
        fig=fig2,
        sub=324,
        zen_angle=za_EW,
        map_slice=rf1YY_EW[0],
        map_error=rf1YY_EW[1],
        model_slice=YY_EW_slice,
        delta_pow=del_pow_ref1_YY_EW,
        pow_fit=fit_ref1_YY_EW,
        slice_label="ref1YY EW",
        model_label="FEE YY EW",
        ylabel=False,
    )

    ax11 = plt_null_test(
        fig=fig2,
        sub=325,
        zen_angle=za_NS,
        del_pow=ref01_YY_NS,
        del_err=error_ref01_YY_NS,
        del_beam=beam_ref01_YY_NS,
        del_fit=fit_ref01_YY_NS,
        null_label="NS rf0-rf1",
        beam_label="FEE Null",
        fit_label="Fit rf0-rf1",
    )

    ax12 = plt_null_test(
        fig=fig2,
        sub=326,
        zen_angle=za_EW,
        del_pow=ref01_YY_EW,
        del_err=error_ref01_YY_EW,
        del_beam=beam_ref01_YY_EW,
        del_fit=fit_ref01_YY_EW,
        null_label="EW rf0-rf1",
        beam_label="FEE Null",
        fit_label="Fit rf0-rf1",
        ylabel=False,
    )

    plt.tight_layout()
    fig2.savefig(f"{out_dir}/null_test_YY_slices.png")



import json
import matplotlib
import numpy as np

matplotlib.use("Agg")
import seaborn as sns
import matplotlib.pyplot as plt


def point_integration(f_name, out_dir, int_thresh):
    """Bin data to calculate integration at each pointing"""

    with open(f"{out_dir}/{f_name}", "r") as data:
        pointings = json.load(data)
        grid_pt = pointings["grid_pt"]
        obs_length = pointings["obs_length"]

    # find unique pointings.
    unique_pointings = list(set(grid_pt))
    unique_pointings = sorted(unique_pointings)

    pointings = []
    integrations = []

    for i in unique_pointings:
        pointings.append(i)
        time = 0
        for j in range(len(grid_pt)):
            if i == grid_pt[j]:
                time = time + obs_length[j]
        integrations.append(time)

    int_hours = np.asarray(integrations) / (60 * 60)

    time_point = []
    point = []

    # time threshold at pointing in hours
    point_threshold = int_thresh

    for i in range(len(int_hours)):
        if int_hours[i] >= point_threshold:
            point.append(pointings[i])
            time_point.append(int_hours[i])

    return (time_point, point)


def pointing_hist(time_point, point, out_dir):
    """Pointing histogram"""

    x = range(len(time_point))
    leg = [int(i) for i in time_point]

    plt.style.use("seaborn")
    fig, ax = plt.subplots(figsize=(8, 6))
    pal = sns.cubehelix_palette(
        len(time_point), start=0.4, rot=-0.5, dark=0.4, reverse=True
    )
    barplot = plt.bar(x, time_point, color=sns.color_palette(pal))

    def autolabel(rects):
        for idx, rect in enumerate(barplot):
            height = rect.get_height()
            ax.text(
                rect.get_x() + rect.get_width() / 2.0,
                height,
                leg[idx],
                ha="center",
                va="bottom",
                rotation=0,
            )

    autolabel(barplot)

    plt.xticks(x, point)
    plt.ylabel("Hours")
    plt.xlim(-0.7, len(time_point) - 0.3)
    plt.xlabel("MWA Grid Pointing Number")
    plt.title("Integration at MWA Grid Pointings")
    plt.tight_layout()
    plt.savefig(f"{out_dir}/pointing_integration.png")


if __name__ == "__main__":

    import argparse

    parser = argparse.ArgumentParser(
        description="""
            Reads ultimate_pointing_times.json, and plots
            a histogram of total integration time at various
            pointings.
            """
    )

    parser.add_argument(
        "--out_dir",
        metavar="\b",
        default="./../../outputs/beam_pointings/",
        help="Directory where json metadata lives. Default=./../../outputs/beam_pointings/",
    )
    parser.add_argument(
        "--f_name",
        metavar="\b",
        default="ultimate_pointing_times.json",
        help="File name of json to be plotted. Default=ultimate_pointing_times.json",
    )
    parser.add_argument(
        "--int_thresh",
        metavar="\b",
        default=200,
        type=int,
        help="Pointing integration time threshold. Default=200",
    )

    args = parser.parse_args()
    out_dir = args.out_dir
    f_name = args.f_name
    int_thresh = args.int_thresh

    time_point, point = point_integration(f_name, out_dir, int_thresh)
    pointing_hist(time_point, point, out_dir)
